(* file: binaryTrees.ml
   author: Bob Muller

   CS1103 Computer Science I Honors

   Lecture 1 of Week 15 (final week)

   This file contains material for reviewing binary trees and tree traversals.
*)

type t = A | B | C | D | E

type btree = Leaf of t
             | Node of {info : t; left : btree; right : btree}

let t0 = Node{info=B; left = Leaf D; right = Leaf E}
let t1 = Node{info=A; left = t0; right = Leaf C}

(* inorder : btree -> t list
 *)
let rec inorder bt =
  match bt with
  | Leaf x -> [x]
  | Node{info; left; right} ->
    (inorder left) @ info :: (inorder right)

(* preorder : btree -> t list
 *)
let rec preorder bt =
  match bt with
  | Leaf x -> [x]
  | Node{info; left; right} ->
    info :: (preorder left) @ (preorder right)

(* postorder : btree -> t list
 *)
let rec postorder bt =
  match bt with
  | Leaf x -> [x]
  | Node{info; left; right} ->
    (postorder left) @ (postorder right) @ [info]

(* levelorder : btree -> t list
 *)
let levelorder bt =
  let rec repeat bts =
    match bts with
    | [] -> []
    | (Leaf t) :: bts -> t :: repeat bts
    | (Node{info; left; right}) :: bts ->
      info :: repeat (bts @ [left; right])
  in
  repeat [bt]
